/**
 * MonCO2meters.cpp is part of bmsapp.
 *
 * bmsapp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * bmsapp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "MonCO2meters.h"
#include "EditSupplier.h"
#include "MainWindow.h"
#include "config.h"



/*
 * Build the table and buttons on the mainscreen.
 * Don't use a ui file, do it dynamicly.
 */
MonCO2meters::MonCO2meters(QWidget *parent) : QDialog(parent)
{
    qDebug() << "MonCO2meters start";

    gridLayout = new QGridLayout(this);
    gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
    tableCO2meters = new QTableWidget(this);
    tableCO2meters->setObjectName(QString::fromUtf8("tableCO2meters"));
    tableCO2meters->setEnabled(true);
    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(tableCO2meters->sizePolicy().hasHeightForWidth());
    tableCO2meters->setSizePolicy(sizePolicy);
    tableCO2meters->setMinimumSize(QSize(1054, 0));
    gridLayout->addWidget(tableCO2meters, 0, 0, 1, 1);

    groupBox = new QGroupBox(this);
    groupBox->setObjectName(QString::fromUtf8("groupBox"));
    groupBox->setEnabled(true);
    groupBox->setFlat(false);
    horizontalLayout = new QHBoxLayout(groupBox);
    horizontalLayout->setSpacing(6);
    horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
    horizontalLayout->setContentsMargins(0, 0, 0, 0);
    quitButton = new QPushButton(groupBox);
    quitButton->setObjectName(QString::fromUtf8("quitButton"));
    quitButton->setMinimumSize(QSize(80, 24));
    quitButton->setText(tr("Quit"));
    QIcon icon;
    icon.addFile(QString::fromUtf8(":icons/silk/door_out.png"), QSize(), QIcon::Normal, QIcon::Off);
    quitButton->setIcon(icon);
    horizontalLayout->addWidget(quitButton, 0, Qt::AlignCenter);
    gridLayout->addWidget(groupBox, 1, 0, 1, 1);

    connect(quitButton, SIGNAL(clicked()), parent, SLOT(fromMonCO2meters()));
    connect(this, SIGNAL(setStatus(QString)), parent, SLOT(statusMsg(QString)));
    connect(parent, SIGNAL(updateCO2meters(QString)), this, SLOT(refreshCO2meters(QString)));
    emit refreshTable();
}


void MonCO2meters::refreshTable()
{
    QTableWidgetItem *item;

    qDebug() << "MonCO2meters reload";
    QSqlQuery query("SELECT record,node,alias,online,mode,beercode,beername,temperature,pressure_bar FROM mon_co2meters ORDER BY node,alias");
    const QStringList labels({tr("Node"), tr("Unit"), tr("Status"), tr("Beer"), tr("Temperature"), tr("Pressure"), tr("Details")});

    this->tableCO2meters->setColumnCount(7);
    this->tableCO2meters->setColumnWidth(0, 150);	/* Node		*/
    this->tableCO2meters->setColumnWidth(1, 100);	/* Unit		*/
    this->tableCO2meters->setColumnWidth(2, 100);	/* Status	*/
    this->tableCO2meters->setColumnWidth(3, 390);	/* Beer		*/
    this->tableCO2meters->setColumnWidth(4, 100);	/* Temperature	*/
    this->tableCO2meters->setColumnWidth(5, 100);	/* Pressure	*/
    this->tableCO2meters->setColumnWidth(6,  90);	/* Edit button	*/
    this->tableCO2meters->setRowCount(query.size());
    this->tableCO2meters->setHorizontalHeaderLabels(labels);
    this->tableCO2meters->verticalHeader()->hide();
    /* Set the widget size to 1054 x 575 in the ui. */

    query.first();
    for (int i = 0 ; i < query.size() ; i++ ) {
	this->tableCO2meters->setItem(i, 0, new QTableWidgetItem(query.value("node").toString()));
	this->tableCO2meters->setItem(i, 1, new QTableWidgetItem(query.value("alias").toString()));

	if (query.value("online").toInt()) {
            item = new QTableWidgetItem(QString("Ok"));
        } else {
            item = new QTableWidgetItem(QString("Offline"));
            item->setForeground(QBrush(QColor(Qt::red)));
        }
        item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
        this->tableCO2meters->setItem(i, 2, item);

	if (query.value("mode").toString() == "ON") {
            item = new QTableWidgetItem(query.value("beercode").toString()+" - "+query.value("beername").toString());
            this->tableCO2meters->setItem(i, 3, item);
        } else {
            this->tableCO2meters->setItem(i, 3, new QTableWidgetItem(QString("")));
        }

	if (query.value("online").toInt()) {
	    item = new QTableWidgetItem(QString("%1°C").arg(query.value("temperature").toDouble(), 4, 'f', 3, '0'));
            item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
            this->tableCO2meters->setItem(i, 4, item);
	    item = new QTableWidgetItem(QString("%1 bar").arg(query.value("pressure_bar").toDouble(), 2, 'f', 1, '0'));
            item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
            this->tableCO2meters->setItem(i, 5, item);
	} else {
	    this->tableCO2meters->setItem(i, 4, new QTableWidgetItem(QString("")));
	    this->tableCO2meters->setItem(i, 5, new QTableWidgetItem(QString("")));
	}

	/* Add the Edit button */
	QWidget* pWidget = new QWidget();
	QPushButton* btn_edit = new QPushButton();
	btn_edit->setObjectName(QString("%1").arg(query.value(0).toString()));	/* Send record with the button */
	btn_edit->setText(tr("Details"));
	connect(btn_edit, SIGNAL(clicked()), this, SLOT(on_editButton_clicked()));
	QHBoxLayout* pLayout = new QHBoxLayout(pWidget);
	pLayout->addWidget(btn_edit);
	pLayout->setContentsMargins(5, 0, 5, 0);
	pWidget->setLayout(pLayout);
	this->tableCO2meters->setCellWidget(i, 6, pWidget);
	query.next();
    }
    emit setStatus(QString(tr("Total items: %1")).arg(query.size()));
}


MonCO2meters::~MonCO2meters() {}


void MonCO2meters::refreshCO2meters(QString data)
{
    qDebug() << "refreshCO2meters" << data;
    emit refreshTable();
}


void MonCO2meters::edit(int recno)
{
//    EditSupplier dialog(recno, this);
    /* Signal from editor if a refresh is needed */
//    connect(&dialog, SIGNAL(entry_changed()), this, SLOT(refreshTable()));
//    dialog.setModal(true);
//    dialog.exec();
}


void MonCO2meters::on_editButton_clicked()
{
    QPushButton *pb = qobject_cast<QPushButton *>(QObject::sender());
    int recno = pb->objectName().toInt();
    edit(recno);
}


